/**
 * JMSSOAPListener
 * @Author:  Naveen Balani
 */

package naveen.jms;

import java.io.*;
import org.apache.axis.AxisEngine;
import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.configuration.FileProvider;
import org.apache.axis.server.AxisServer;
import org.apache.axis.message.SOAPEnvelope;
import org.apache.axis.message.SOAPFaultElement;

import javax.xml.rpc.namespace.QName;
import org.apache.axis.Constants;
import java.util.Properties;
import javax.jms.*;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.apache.axis.message.SOAPEnvelope;

import org.w3c.dom.Element;
import org.apache.axis.utils.XMLUtils;

public class JMSSOAPListener implements MessageListener {

	//Define JMS Queue family classes
	private QueueConnectionFactory queueConnFactory;
	private QueueConnection queueConn;
	private QueueSession session;
	private QueueSender sender;
	private QueueReceiver receiver;
	private Queue queue;
	private boolean quit = false;

	// Define TextMessage for wrapping the SOAP (request) Envelope
	private TextMessage jmsMsg;

	private AxisEngine engine = null;
	private String factoryName;

	private String transportName;

	javax.jms.Message msg;

	public JMSSOAPListener() {
	}

	public AxisEngine getAxisEngine() throws Exception {

		if (engine != null) {
			return engine;

		}

		FileProvider provider = new FileProvider(configuration);
		return new AxisServer(provider);
	}

	// This method is invoked when ever a new message arrives in the Response QUEUE.
	public void onMessage(javax.jms.Message msg) {

		System.out.println("onMessage event Executed");
		quit = true;
		MessageContext msgContext = null;
		org.apache.axis.Message responseMsg = null;

		try {
			String msgTxt = null;
			TextMessage jmsMsg = (TextMessage) msg;
			System.out.println(jmsMsg.getText());
			Queue replyQueue = (Queue) msg.getJMSReplyTo();
			sender = session.createSender(replyQueue);

			// Create an instance of AxisServer from configuration file
			AxisEngine engine = getAxisEngine();

			// Create a MessageContext and associate with the engine
			msgContext = new MessageContext(engine);

			// Wrap the incoming TextMessage into Axis Message
			org.apache.axis.Message soapMessage = 
				new org.apache.axis.Message(jmsMsg.getText()); 

			System.out.println("JMS-SOAP Request:");

			System.out.println(jmsMsg.getText() + "\n");

			// Set the request message in MessageContext
			msgContext.setRequestMessage(soapMessage);

			// Invoke AxisEngine (AxisServer)
			engine.invoke(msgContext);

			System.out.println("---Getting response message from Price WebService");

			// Get the response message from the MessageContext
			responseMsg = msgContext.getResponseMessage();

			// Get a String representation of response
			SOAPEnvelope envelope = responseMsg.getSOAPEnvelope();
			Element envElement = envelope.getAsDOM();
			String strSOAPBody = XMLUtils.ElementToString(envElement);

			System.out.println("JMS-SOAP Response:");
			System.out.println(strSOAPBody);

			// Create an instance of TextMessage to wrap the SOAP (response)
			TextMessage jmsResponseMsg = session.createTextMessage();
			jmsResponseMsg.setText(strSOAPBody);

			// Put the response into the queue: 'replyQueue'
			System.out.println("\n---Sending JMS-SOAP Response to the Response Queue");

			sender.send(jmsResponseMsg);

			System.out.println("Waiting for Next JMS-SOAP Requests");

		} catch (Exception e) {
			e.printStackTrace();
			if (!(e instanceof AxisFault)) {
				e = AxisFault.makeFault(e);
			}
			responseMsg = msgContext.getResponseMessage();
			if (responseMsg == null) {
				responseMsg = new org.apache.axis.Message((AxisFault) e);
				msgContext.setResponseMessage(responseMsg);
			} else {
				try {
					SOAPEnvelope env = responseMsg.getSOAPEnvelope();
					env.clearBody();
					env.addBodyElement(new SOAPFaultElement((AxisFault) e));
				} catch (AxisFault af) {
					af.printStackTrace();
				}
			}
		}
	}

	public static void main(String args[]) throws Exception {
		JMSSOAPListener jmsListener = new JMSSOAPListener();
		jmsListener.initialize();

		//Syncronize JMS Listener
		synchronized (jmsListener) {
			// Wait until a "Quit" Message is recieved
			while (!jmsListener.quit) {
				try {
					jmsListener.wait();
				} catch (InterruptedException ie) {
				}
			}
		}
	}
	//Configuration file : Change this configuration file to point it accordindly to your
	//WSDK AXIS Installation
	private static final String configuration = 
		"C:\\WSDK\\WebSphere\\installedApps\\wsdk.ear\\axis.war\\WEB-INF\\server-config.wsdd"; 
	//JMS Factory configuration 

	// Defines the JMS context factory.
	public static final String JMS_FACTORY = "MQ_JMS_MANAGER"; 
	// Defines the JNDI context factory.
	public static final String JNDI_FACTORY = 
		"com.sun.jndi.fscontext.RefFSContextFactory"; 
	public static final String providerURL = "file:/C:/JNDI-Directory"; 
	// Defines the queue.
	public static final String REQUEST_QUEUE = "JMS_RequestQueue";
	public static final InitialContext getInitialContext()
		throws javax.naming.NamingException {

		Properties props = new Properties();
		props.put(javax.naming.Context.PROVIDER_URL, providerURL);
		props.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
		InitialContext ictx = new javax.naming.InitialContext(props);
		return ictx;
	}
	private void initialize() throws AxisFault {

		try {
			InitialContext ictx = getInitialContext();
			queueConnFactory = (QueueConnectionFactory) ictx.lookup(JMS_FACTORY);

			// Look up the Request Queue
			queue = (Queue) ictx.lookup(REQUEST_QUEUE);
			queueConn = queueConnFactory.createQueueConnection();

			// Create a session (non-transactional) that automatically acknowledges
			//message receipt.
			session = queueConn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

			receiver = session.createReceiver(queue);
			queueConn.start();

			System.out.println("JMSSOAPListener ready to receive messages ...\n");

			//Fire the onMesage event when message arrive at the Request Queue
			receiver.setMessageListener(this);

		} catch (Exception e) {

			// System.out.println(e);
			throw AxisFault.makeFault(e);
		}
	}
}