/**
 * JMSSOAPSender
 * @Author:  Naveen Balani
 */

package naveen.jms;

import org.apache.axis.AxisFault;
import org.apache.axis.Message;
import org.apache.axis.MessageContext;
import org.apache.axis.encoding.Base64;
import org.apache.axis.handlers.BasicHandler;
import org.apache.axis.message.MessageElement;
import org.apache.axis.utils.JavaUtils;

import org.apache.axis.message.SOAPEnvelope;
import org.w3c.dom.Element;
import org.apache.axis.utils.XMLUtils;

import javax.jms.*;
import javax.naming.InitialContext;

import java.util.Properties;
public class JMSSOAPSender extends BasicHandler {

	//Define JMS Queue family classes
	private QueueConnectionFactory queueConnFactory;
	private QueueConnection queueConn;
	private QueueSession session;
	private QueueSender sender;
	private QueueReceiver receiver;
	private Queue queue;

	//Define TextMessage for wrapping the SOAP (request) Envelope
	private TextMessage jmsMsg;

	public JMSSOAPSender() {
		try {

			System.out.println("JMSSOAP Sender called");

			// Get the initial context to lookup the AdministeredObjects
			InitialContext ictx = getInitialContext();

			// Initialize Queue related objects.
			queueConnFactory = (QueueConnectionFactory) ictx.lookup(JMS_FACTORY);
			queueConn = queueConnFactory.createQueueConnection();
			session = queueConn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);

			//Lookup Request Queue
			queue = (Queue) ictx.lookup(REQUEST_QUEUE);

			//Create sender object
			sender = session.createSender(queue);

			// Create an object of TextMessage
			jmsMsg = session.createTextMessage();

			// Create a reponse queue for receiving the SOAP (response) Envelope
			Queue queue1 = (Queue) ictx.lookup(RESPONSE_QUEUE);
			jmsMsg.setJMSReplyTo(queue1);
			receiver = session.createReceiver(queue1);

			queueConn.start();

		} catch (Exception e) {
			System.out.println(e);
			System.exit(0);
		}
	}

	// Axis Engine invokes this method
	public void invoke(MessageContext msgContext) throws AxisFault {
		try {

			// Get the request Message from MessageContext
			org.apache.axis.Message axisMsg = msgContext.getRequestMessage();

			// Get a string representation of SOAP Envelope
			SOAPEnvelope soapEnvelope = axisMsg.getSOAPEnvelope();
			Element envElement = soapEnvelope.getAsDOM();
			String strSOAPBody = XMLUtils.ElementToString(envElement);

			System.out.println("JMS-SOAP Request:\n");
			System.out.println(strSOAPBody + "\n");

			// Set SOAP Envelope as the text message
			jmsMsg.setText(strSOAPBody);

			System.out.println("Sending JMS-SOAP Request to Request Queue");
			sender.send(jmsMsg);
			System.out.println("Sent JMS-SOAP Request to Response Queue \n");

			//receiver = session.createReceiver(queue1);
			System.out.println("Waiting to Receive JMS-SOAP Response from Response Queue");
			TextMessage replyMsg = (TextMessage) receiver.receive();
			System.out.println("Received JMS-SOAP Response from Response Queue\n");
			System.out.println("JMS-SOAP Response:");
			System.out.println("\n");
			System.out.println("" + replyMsg.getText());
			System.out.println("\n");

			org.apache.axis.Message responseMsg = 
				new org.apache.axis.Message(replyMsg.getText()); 
			msgContext.setResponseMessage(responseMsg);
			System.out.println("End of JMSSOAPSender\n");
		} catch (Exception e) {
			throw AxisFault.makeFault(e);
		}
	}
	public void cleanUp() {
		try {
			sender.close();
			receiver.close();
			session.close();
			queueConn.close();
		} catch (Exception e) {
			System.out.println(e);
			System.exit(0);
		}
	}
	//JMS Factory configuration 

	// Defines the JMS context factory.
	public static final String JMS_FACTORY = "MQ_JMS_MANAGER"; 
	// Defines the JNDI context factory.
	public static final String JNDI_FACTORY = 
		"com.sun.jndi.fscontext.RefFSContextFactory"; 
	public static final String providerURL = "file:/C:/JNDI-Directory"; 
	// Defines the queue.
	public static final String REQUEST_QUEUE = "JMS_RequestQueue";
	public static final String RESPONSE_QUEUE = "JMS_ResponseQueue";
	public static final InitialContext getInitialContext()
		throws javax.naming.NamingException {

		Properties props = new Properties();
		props.put(javax.naming.Context.PROVIDER_URL, providerURL);
		props.put(javax.naming.Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY);
		InitialContext ictx = new javax.naming.InitialContext(props);
		return ictx;
	}
}